-------------------------------------------------------------------------------
-- Localized Lua globals
-------------------------------------------------------------------------------
local _G = getfenv(0)

local string = _G.string

local pairs = _G.pairs
local wipe = _G.table.wipe 

-------------------------------------------------------------------------------
-- AddOn namespace
-------------------------------------------------------------------------------
local MODNAME = "Syllabus"
local Syllabus = LibStub("AceAddon-3.0"):NewAddon(MODNAME, "AceEvent-3.0", "AceConsole-3.0")
_G[MODNAME] = Syllabus

local LDB = LibStub("LibDataBroker-1.1")
local QTip = LibStub("LibQTip-1.0")

local dev = false
--[===[@debug@
dev = true
--@end-debug@]===]
local L = LibStub("AceLocale-3.0"):GetLocale(MODNAME, "enUS", true, dev)

local DataObj
local tooltip

local version = GetAddOnMetadata(MODNAME, "Version")
version = string.gsub(version, "@project.version@", " - Development")

-------------------------------------------------------------------------------
-- Constants
-------------------------------------------------------------------------------
local MY_CLASS = UnitClass("player")

-------------------------------------------------------------------------------
-- Variables
-------------------------------------------------------------------------------
local update_queued = false
local in_combat = false
local skill_ranks = {}

local db

-------------------------------------------------------------------------------
-- Functions
-------------------------------------------------------------------------------
local function UpdateSkills()
	if in_combat then
		update_queued = true
		return
	end
	wipe(skill_ranks)

	for i = 1, GetNumSkillLines() do
		local name, is_header, _, rank, _, _, max_rank,	_, _, _, _, _, _ = GetSkillLineInfo(i);

		if not is_header and not string.find(name, L["Language"]) then
			--[===[@debug@
			Syllabus:Print("Adding "..name)
			--@end-debug@]===]
			skill_ranks[name] = skill_ranks[name] or {}
			skill_ranks[name].rank = rank
			skill_ranks[name].max_rank = max_rank
			db.char.known[name] = true
		end
	end
end


-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------
function Syllabus:PLAYER_REGEN_DISABLED()
	in_combat = true
end

function Syllabus:PLAYER_REGEN_ENABLED()
	in_combat = false

	if update_queued then
		update_queued = false
		UpdateSkills()
	end
end

function Syllabus:TRADE_SKILL_CLOSE()
	if IsTradeSkillLinked() then	-- We don't want to save the information if this isn't our tradeskill list.
		return
	end

	for i = 1, GetNumTradeSkills() do
		local name, type, _, _, _ = GetTradeSkillInfo(i)

		if name and type ~= "header" then
			Syllabus:Print("Adding "..name)
			db.char.known[name] = true
		end
	end
end

do
	local type_filters = {
		["available"] = 0,
		["unavailable"] = 0,
		["used"] = 0,
	}
	local current_trainer = {}

	local DataMiner = CreateFrame("GameTooltip", "SyllabusDataMiner", UIParent, "GameTooltipTemplate")
	DataMiner:SetOwner(UIParent, "ANCHOR_NONE")

	function Syllabus:TRAINER_SHOW()
		GameTooltip_SetDefaultAnchor(DataMiner, UIParent)
		DataMiner:SetUnit("npc")

		local trainer_name = _G["SyllabusDataMinerTextLeft1"]:GetText()
		current_trainer.role = _G["SyllabusDataMinerTextLeft2"]:GetText()
		DataMiner:Hide()

		current_trainer.x, current_trainer.y = GetPlayerMapPosition("player")
		current_trainer.location = GetRealZoneText()

		self:Print(string.format("Trainer is: %s <%s>, in %s at %.2f, %.2f.", trainer_name, current_trainer.role, current_trainer.location, current_trainer.x * 100, current_trainer.y * 100))

		-- Save the user's preferences before opening everything.
		for k, v in pairs(type_filters) do
			type_filters[k] = GetTrainerServiceTypeFilter(k)
			SetTrainerServiceTypeFilter(k, 1)
		end

		for i = 1, GetNumTrainerServices() do
			local _ , _, category = GetTrainerServiceInfo(i)

			if not category then
				break
			end

			if category == "header" then
				ExpandTrainerSkillLine(i)
			end
		end
		local trainer_db = db.global.trainer
		local skill_db = db.global.skill
		local trainer_type = IsTradeskillTrainer() and GetTrainerServiceSkillLine(2) or MY_CLASS

		trainer_db[trainer_type] = trainer_db[trainer_type] or {}
		trainer_db[trainer_type][trainer_name] = current_trainer

		skill_db[trainer_type] = skill_db[trainer_type] or {}
		skill_db[trainer_type].category = skill_db[trainer_type].category or {}
		skill_db[trainer_type].skill = skill_db[trainer_type].skill or {}

		-- Restore the previous values for the user's type filters.
		for k, v in pairs(type_filters) do
			SetTrainerServiceTypeFilter(k, type_filters[k])
		end
	end
end	-- do

-------------------------------------------------------------------------------
-- Initialization functions
-------------------------------------------------------------------------------
function Syllabus:OnInitialize()
	-- Hack to get GetPlayerMapPosition() to return correct values for 3.0.x
	local defaults = {
		global = {
			category = {},
			skill = {},
			trainer = {},
			-- Configuration data
			show = {
				cost = true,
				known = false,
				not_trainable = false,
				overall_cost = true,
				requirements = true,
				skill_count = true,
				trainable = true,
				trainable_future = true,
				trainer_names = true,
			},
			level_range = 2,
			skill_range = 5,
		},
		char = {
			known = {},
			hidden = {},
		},
	}
	db = LibStub("AceDB-3.0"):New(MODNAME.."DB", defaults)
--	self:SetupOptions()
	self.OnInitialize = nil
end

function Syllabus:OnEnable()
	UpdateSkills()
	self:RegisterEvent("PLAYER_LEVEL_UP", UpdateSkills)
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("SKILL_LINES_CHANGED", UpdateSkills)
	self:RegisterEvent("TRADE_SKILL_CLOSE")
	self:RegisterEvent("TRAINER_SHOW")
end

function Syllabus:OnDisable()
	self:UnregisterEvent("PLAYER_LEVEL_UP")
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	self:UnregisterEvent("SKILL_LINES_CHANGED")
	self:UnregisterEvent("TRADE_SKILL_CLOSE")
	self:UnregisterEvent("TRAINER_SHOW")
end
